import React, { useState, useRef, useMemo, useImperativeHandle, forwardRef, useEffect } from 'react';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import html2canvas from 'html2canvas';
import { CustomChartPlotly, plotTypeOptions } from './components/CustomChartPlotly';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { EntityPill } from '@graphpolaris/shared/lib/components/pills/Pill';
import { Button } from '@graphpolaris/shared/lib/components/buttons';

export interface Vis1DProps {
  plotType: (typeof plotTypeOptions)[number]; // plotly plot type
  title: string; // title of the plot
  nodeLabel: string; // node label to plot
  xAxisLabel?: string;
  yAxisLabel?: string;
  showAxis: boolean;
}

const defaultSettings: Vis1DProps = {
  plotType: 'bar',
  title: '',
  nodeLabel: '',
  xAxisLabel: '',
  yAxisLabel: '',
  showAxis: true,
};

export interface Vis1DVisHandle {
  exportImageInternal: () => void;
}

const Vis1D = forwardRef<Vis1DVisHandle, VisualizationPropTypes<Vis1DProps>>(({ data, settings }, refExternal) => {
  const internalRef = useRef<HTMLDivElement>(null);

  useImperativeHandle(refExternal, () => ({
    exportImageInternal() {
      const captureImage = () => {
        const element = internalRef.current;
        if (element) {
          html2canvas(element, {
            backgroundColor: '#FFFFFF',
          })
            .then((canvas) => {
              const finalImage = canvas.toDataURL('image/png');
              const link = document.createElement('a');
              link.href = finalImage;
              link.download = 'Vis1D.png';
              document.body.appendChild(link);
              link.click();
              document.body.removeChild(link);
            })
            .catch((error) => {
              console.error('Error capturing image:', error);
            });
        } else {
          console.error('Container element not found');
        }
      };

      const renderCanvas = () => {
        requestAnimationFrame(() => {
          captureImage();
        });
      };

      renderCanvas();
    },
  }));

  const getAttributeValues = (attributeKey: string | number | undefined) => {
    if (!settings.nodeLabel || !attributeKey) {
      return [];
    }

    return data.nodes
      .filter((item) => item.label === settings.nodeLabel && item.attributes && attributeKey in item.attributes)
      .map((item) => item.attributes[attributeKey]);
  };

  const xAxisData = useMemo(() => getAttributeValues(settings.xAxisLabel), [data, settings.nodeLabel, settings.xAxisLabel]);
  const yAxisData = useMemo(() => getAttributeValues(settings.yAxisLabel), [data, settings.nodeLabel, settings.yAxisLabel]);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden" ref={internalRef}>
      <CustomChartPlotly
        xAxisData={xAxisData as string[] | number[]}
        yAxisData={yAxisData as string[] | number[]}
        plotType={settings.plotType}
        title={settings.title}
        showAxis={settings.showAxis}
        xAxisLabel={settings.xAxisLabel}
        yAxisLabel={settings.yAxisLabel}
      />
    </div>
  );
});

const Vis1DSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<Vis1DProps>) => {
  const mutablePlotTypes = [...plotTypeOptions];
  const [attributeOptions, setAttributeOptions] = useState<string[]>([]);

  useEffect(() => {
    if (settings.nodeLabel === '' && graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      const nodeLabelTemp = graphMetadata.nodes.labels[0];
      updateSettings({ nodeLabel: nodeLabelTemp });
    }
  }, [settings.nodeLabel, graphMetadata]);

  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0 && settings.nodeLabel != '') {
      const newAttributeOptions = Object.keys(graphMetadata.nodes.types[settings.nodeLabel].attributes);
      if (settings.xAxisLabel === '') {
        updateSettings({ xAxisLabel: newAttributeOptions[0] });
      }
      // initialize the selected option for creating the dropdown and plots
      setAttributeOptions(newAttributeOptions);
    }
  }, [graphMetadata, settings.nodeLabel]);

  return (
    <SettingsContainer>
      <div className="p-1">
        <Input
          className="w-full text-justify justify-start mb-2"
          type="dropdown"
          value={settings.nodeLabel}
          options={graphMetadata.nodes.labels}
          onChange={(val) => updateSettings({ nodeLabel: val as string })}
          overrideRender={
            <EntityPill
              title={
                <div className="flex flex-row justify-between items-center cursor-pointer">
                  <span>{settings.nodeLabel || ''}</span>
                  <Button variantType="secondary" variant="ghost" size="2xs" iconComponent="icon-[ic--baseline-arrow-drop-down]" />
                </div>
              }
            />
          }
        />
        <div className="mb-2">
          <Input type="text" label="Title" value={settings.title} onChange={(value) => updateSettings({ title: value as string })} />
        </div>
        <div className="mb-2">
          <Input
            type="dropdown"
            label="Chart"
            value={settings.plotType}
            options={mutablePlotTypes}
            onChange={(value: string | number) => {
              updateSettings({ plotType: value as (typeof plotTypeOptions)[number] });
            }}
          />
        </div>
        <div className="mb-2">
          <Input
            type="dropdown"
            label="X-axis:"
            value={settings.xAxisLabel}
            options={attributeOptions}
            onChange={(value) => {
              updateSettings({ xAxisLabel: value as string });
            }}
          />
        </div>
        {(settings.plotType === 'scatter' || settings.plotType === 'line') && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Y-axis:"
              value={settings.yAxisLabel}
              options={attributeOptions}
              onChange={(value) => {
                updateSettings({ yAxisLabel: value as string });
              }}
            />
          </div>
        )}
        <div className="mb-2">
          <Input type="boolean" label="Show axis" value={settings.showAxis} onChange={(val) => updateSettings({ showAxis: val })} />
        </div>
      </div>
    </SettingsContainer>
  );
};

const Vis1DRef = React.createRef<Vis1DVisHandle>();

export const Vis1DComponent: VISComponentType<Vis1DProps> = {
  component: React.forwardRef((props: VisualizationPropTypes<Vis1DProps>, ref) => <Vis1D {...props} ref={Vis1DRef} />),
  settingsComponent: Vis1DSettings,
  settings: defaultSettings,
  exportImage: () => {
    if (Vis1DRef.current) {
      Vis1DRef.current.exportImageInternal();
    } else {
      console.error('1Dvis reference is not set.');
    }
  },
};

export default Vis1DComponent;
